import { useEffect, useState } from "react";
import {  useThree } from "@react-three/fiber";
import * as THREE from "three";
import { QuadraticBezierLine } from "@react-three/drei";
import { usePlayButtonStore } from "../../../../store/usePlayButtonStore";
import { Arrows } from "../../../simulation/events/arrows/arrows";
import { useProductContext } from "../../../simulation/products/productContext";
import { useDuplicateProductStore } from "../../duplicateStores/duplicateProductStore";

interface ConnectionLine {
    id: string;
    startPointUuid: string;
    endPointUuid: string;
    trigger: TriggerSchema;
}

function TriggerDuplicate() {
    const { scene } = useThree();
    const { selectedProductStore } = useProductContext();
    const { products, getProductById } = useDuplicateProductStore();
    const { selectedProduct } = selectedProductStore();
    const { isPlaying } = usePlayButtonStore();

    const [connections, setConnections] = useState<ConnectionLine[]>([]);

    useEffect(() => {
        const newConnections: ConnectionLine[] = [];

        const product = getProductById(selectedProduct.productUuid);
        if (!product || products.length === 0) return;

        product.eventDatas.forEach(event => {
            // Handle Conveyor points
            if (event.type === "transfer" && 'points' in event) {
                event.points.forEach(point => {
                    if (point.action?.triggers) {
                        point.action.triggers.forEach(trigger => {
                            if (trigger.triggeredAsset && trigger.triggeredAsset.triggeredPoint) {
                                newConnections.push({
                                    id: `${point.uuid}-${trigger.triggeredAsset.triggeredPoint.pointUuid}-${trigger.triggerUuid}`,
                                    startPointUuid: point.uuid,
                                    endPointUuid: trigger.triggeredAsset.triggeredPoint.pointUuid,
                                    trigger
                                });
                            }
                        });
                    }
                });
            }
            // Handle Vehicle point
            else if (event.type === "vehicle" && 'point' in event) {
                const point = event.point;
                if (point.action?.triggers) {
                    point.action.triggers.forEach(trigger => {
                        if (trigger.triggeredAsset && trigger.triggeredAsset.triggeredPoint) {
                            newConnections.push({
                                id: `${point.uuid}-${trigger.triggeredAsset.triggeredPoint.pointUuid}-${trigger.triggerUuid}`,
                                startPointUuid: point.uuid,
                                endPointUuid: trigger.triggeredAsset.triggeredPoint.pointUuid,
                                trigger
                            });
                        }
                    });
                }
            }
            // Handle Robotic Arm points
            else if (event.type === "roboticArm" && 'point' in event) {
                const point = event.point;
                point.actions?.forEach(action => {
                    action.triggers?.forEach(trigger => {
                        if (trigger.triggeredAsset && trigger.triggeredAsset.triggeredPoint) {
                            newConnections.push({
                                id: `${point.uuid}-${trigger.triggeredAsset.triggeredPoint.pointUuid}-${trigger.triggerUuid}`,
                                startPointUuid: point.uuid,
                                endPointUuid: trigger.triggeredAsset.triggeredPoint.pointUuid,
                                trigger
                            });
                        }
                    });
                });
            }
            // Handle Machine point
            else if (event.type === "machine" && 'point' in event) {
                const point = event.point;
                if (point.action?.triggers) {
                    point.action.triggers.forEach(trigger => {
                        if (trigger.triggeredAsset && trigger.triggeredAsset.triggeredPoint) {
                            newConnections.push({
                                id: `${point.uuid}-${trigger.triggeredAsset.triggeredPoint.pointUuid}-${trigger.triggerUuid}`,
                                startPointUuid: point.uuid,
                                endPointUuid: trigger.triggeredAsset.triggeredPoint.pointUuid,
                                trigger
                            });
                        }
                    });
                }
            }
            // Handle StorageUnit point
            else if (event.type === "storageUnit" && 'point' in event) {
                const point = event.point;
                if (point.action?.triggers) {
                    point.action.triggers.forEach(trigger => {
                        if (trigger.triggeredAsset && trigger.triggeredAsset.triggeredPoint) {
                            newConnections.push({
                                id: `${point.uuid}-${trigger.triggeredAsset.triggeredPoint.pointUuid}-${trigger.triggerUuid}`,
                                startPointUuid: point.uuid,
                                endPointUuid: trigger.triggeredAsset.triggeredPoint.pointUuid,
                                trigger
                            });
                        }
                    });
                }
            }
        });

        setConnections(newConnections);
    }, [products, selectedProduct.productUuid]);

    const getWorldPositionFromScene = (pointUuid: string): THREE.Vector3 | null => {
        const pointObj = scene.getObjectByProperty("uuid", pointUuid);
        if (!pointObj) return null;

        const worldPosition = new THREE.Vector3();
        pointObj.getWorldPosition(worldPosition);
        return worldPosition;
    };

    return (
        <group name="simulationConnectionGroup" visible={!isPlaying}>
            {connections.map((connection) => {
                const startPoint = getWorldPositionFromScene(connection.startPointUuid);
                const endPoint = getWorldPositionFromScene(connection.endPointUuid);

                if (!startPoint || !endPoint) return null;

                const distance = startPoint.distanceTo(endPoint);
                const heightFactor = Math.max(0.5, distance * 0.2);
                const midPoint = new THREE.Vector3(
                    (startPoint.x + endPoint.x) / 2,
                    Math.max(startPoint.y, endPoint.y) + heightFactor,
                    (startPoint.z + endPoint.z) / 2
                );

                return (
                    <QuadraticBezierLine
                        key={connection.id}
                        start={startPoint.toArray()}
                        end={endPoint.toArray()}
                        mid={midPoint.toArray()}
                        color={"#42a5f5"}
                        lineWidth={4}
                        dashed={true}
                        dashSize={0.75}
                        dashScale={20}
                        userData={connection.trigger}
                    />
                );
            })}

            <Arrows connections={connections} />
        </group>
    );
}

export default TriggerDuplicate;